//Copyright (c) 1997 The Open Group
//
//Permission to use, copy, modify, and distribute this software and its documentation for
//any purpose and without fee is hereby granted, provided that the above copyright notice
//appears in all copies and that both the copyright notice and this permission notice
//appear in supporting documentation, and that the name of ("TOG") or The Open Group not
//be used in advertising or publicity pertaining to distribution of the software without
//specific, written prior permission.
//
//THE OPEN GROUP DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL
//IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT
//SHALL TOG BE LIABLE FOR ANY SPECIAL, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
//WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN ACTION OF CONTRACT,
//NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
//PERFORMANCE OF THIS SOFTWARE. 

/* 
 * TopicsDialog
 * by Bill la Forge
 * http://www.opengroup.org/~laforge
 */

package ORG.opengroup.awt;
import ORG.opengroup.misc.*;
import java.awt.*;
import java.awt.event.*;
    

/**
 * Help Topics Dialog. Driven by a file of topics and file names, the user
 * is presented with a list of topics, each of which has an associated text file.
 */
public class TopicsDialog extends ButtonDialog
{
	private Object obj;
	private List topics;
	private String map[];
	private TextArea topic;
	private String selected;

	class ListIL implements ItemListener
	{
		public void itemStateChanged(ItemEvent ie)
		{
			String nsf=topics.getSelectedItem();
			if (nsf==null && selected.equals(""))
				return;
			if (nsf!=null && nsf.equals(selected))
				return;
			if (topic!=null)
			{
				remove(topic);
				topic=null;
			}
			if (nsf==null || nsf.equals(""))
			{
				selected="";
				pack();
				return;
			}
			int i;
			for(i=0;i<map.length;i+=2)
			{
				if (nsf.equals(map[i]))
				{
					selected=nsf;
					String elp;
					try 
					{
			elp=new String(Reads.read(obj.getClass().getResourceAsStream(map[i+1])));
						int j=elp.indexOf("$$$");
						if (j>-1)
						{
							elp=elp.substring(j+3,elp.length());
						}
					} catch (Exception ex) 
					{
						elp="Unable to read "+map[i+1];
					}
					Line ine=new Line(elp);
					topic=new TextArea(elp,ine.getHeight(),
						ine.getWidth(),TextArea.SCROLLBARS_NONE);
					topic.setEditable(false);
					add("East",topic);
					pack();
					return;
				}
			}
			pack();
		}
	}

	/**
	 * .
	 * @param parent		The parent frame.
	 * @param x			x-location of the dialog.
	 * @param y			y-location of the dialog.
	 * @param pobj		The object whose class determines the location of
	 * 				the related help files.
	 */
	public TopicsDialog(
		Frame parent, 
		int x, 
		int y,
		Object pobj)
	{
		super(parent,"Help Topics",x,y,"OK");
		selected="";
		obj=pobj;
		String txt;
		try
		{
			txt=new String(Reads.read(obj.getClass().getResourceAsStream("topics.txt")));
		} catch (Exception eeee)
		{   
			txt="Unable to read topics.txt";
			TextArea text=new TextArea(txt,1,
				txt.length(),TextArea.SCROLLBARS_NONE);
			text.setEditable(false);
			add("Center",text);
			pack();
			return;
		}
		Line lnt=new Line(txt);
		map=new String[lnt.getHeight()];
		int i;
		for (i=0;i<map.length;++i)
			map[i]=(String)lnt.nextElement();
		topics=new List(map.length/2);
		topics.addItemListener(new ListIL());
		for (i=0;i<map.length;i+=2)
		{
			topics.add(map[i]);
		}
		add("West",topics);
		topic=null;
		pack();
	}
}
