//Copyright (c) 1997 The Open Group
//
//Permission to use, copy, modify, and distribute this software and its documentation for
//any purpose and without fee is hereby granted, provided that the above copyright notice
//appears in all copies and that both the copyright notice and this permission notice
//appear in supporting documentation, and that the name of ("TOG") or The Open Group not
//be used in advertising or publicity pertaining to distribution of the software without
//specific, written prior permission.
//
//THE OPEN GROUP DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL
//IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT
//SHALL TOG BE LIABLE FOR ANY SPECIAL, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
//WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN ACTION OF CONTRACT,
//NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
//PERFORMANCE OF THIS SOFTWARE. 

/* 
 * ORG.opengroup.moabatch.Ready version 0.1
 * by Bill la Forge
 * http://www.opengroup.org/~laforge
 */

package ORG.opengroup.beans;

import java.beans.*;
import java.io.*;

/**
 * Ready will call method deserialized when the object is deserialized.
 * <p>
 * The intent here is that a Bean will have a nested class which 
 * extends Ready, overriding the deserialized method. 
 * In the following example example, "Blip!" us printed when
 * TestReady is deserialized:
 * <pre>
 * public class TestReady
 * {
 *     Ready ready = new Ready()
 *     {
 *         public void deserialized()
 *         {
 *             System.out.println("Blip!");
 *         }
 *     };
 * }
 * </pre>
 */
public class Ready implements Serializable
{
    private void writeObject(ObjectOutputStream s)
        throws IOException
    {
        s.defaultWriteObject();
    }

    private void readObject(ObjectInputStream s)
        throws ClassNotFoundException, IOException 
    {
        s.defaultReadObject();
        deserialized();
    }

    /*
     * Override this method to gain control when
     * the object is constructed or deserialized.
     */
    public void deserialized()
    {
    }
}  
