//Copyright (c) 1997 The Open Group
//
//Permission to use, copy, modify, and distribute this software and its documentation for
//any purpose and without fee is hereby granted, provided that the above copyright notice
//appears in all copies and that both the copyright notice and this permission notice
//appear in supporting documentation, and that the name of ("TOG") or The Open Group not
//be used in advertising or publicity pertaining to distribution of the software without
//specific, written prior permission.
//
//THE OPEN GROUP DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL
//IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT
//SHALL TOG BE LIABLE FOR ANY SPECIAL, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
//WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN ACTION OF CONTRACT,
//NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
//PERFORMANCE OF THIS SOFTWARE. 

package ORG.opengroup.misc;
import java.util.*;

/**
 * A structure tp process a multi-line string
 */
public class Line implements Enumeration
{
	private String rem;
	private int height;
	private int width;

	/**
	 * Analyze a multi-line string. Carrage returns are ignored;
	 * only line feeds ('\n') are significant.
	 * @param multi The multi-line string or null.
	 */
	public Line(String multi)
	{
		rem=multi;
		height=0;
		width=0;
		if (rem==null)
			return;
		int x=0;
		int y=0;
		do
		{
			++height;
			y=rem.indexOf('\n',x);
			int d=0;
			if (y==-1)
				d=rem.length()-x;
			else
				d=y-x;
			if (d>width)
				width=d;
			x=y+1;
		} while (x>0);
	}

	/**
	 * Returns the number of lines in the string.
	 * Returns 0 only if null was passed to the constructor.
	 * Returns 1 only if there were no line feeds passed to the constructor.
	 */
	public int getHeight()
	{
		return height;
	}

	/**
	 * Returns the number of characters in the longest line.
	 */
	public int getWidth()
	{
		return width;
	}

	/**
	 * Returns true if there is another line. 
	 */
	public boolean hasMoreElements()
	{
		return rem!=null;
	}

	/**
	 * Returns the next line. (Cast to String.)
	 * If there are no more lines, null is returned.
	 */
	public Object nextElement()
	{
		String rv=rem;
		if (rem==null)
			return rv;
		int x=rem.indexOf('\n');
		if (x==-1)
		{
			rem=null;
			if (rv.endsWith("\r"))
				return rv.substring(0,rv.length()-1);
			return rv;
		}
		rv=rem.substring(0,x);
		rem=rem.substring(x+1,rem.length());
		if (rv.endsWith("\r"))
			return rv.substring(0,rv.length()-1);
		return rv;
	}
}
