//Copyright (c) 1998 The Open Group
//
//Permission to use, copy, modify, and distribute this software and its documentation for
//any purpose and without fee is hereby granted, provided that the above copyright notice
//appears in all copies and that both the copyright notice and this permission notice
//appear in supporting documentation, and that the name of ("TOG") or The Open Group not
//be used in advertising or publicity pertaining to distribution of the software without
//specific, written prior permission.
//
//THE OPEN GROUP DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL
//IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT
//SHALL TOG BE LIABLE FOR ANY SPECIAL, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
//WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN ACTION OF CONTRACT,
//NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
//PERFORMANCE OF THIS SOFTWARE. 

package ORG.opengroup.xface;
import ORG.opengroup.misc.*;
import ORG.opengroup.jar.*;
import java.io.*;
import java.util.*;
import java.beans.*;

/**
 */
public class XSerialization implements Serialization, XFaceFactory, XObjFactory
{
	private XFaceFactory xFaceFactory=new XFaceFactoryImpl();
	private String xFaceFactoryBeanName="XFaceFactoryImpl";
	private XObjFactory xObjFactory=new XObjFactoryImpl();
	private String xObjFactoryBeanName="XObjFactoryImpl";

	private transient byte[] b=null;
	private transient int off=0;
	private transient ClassLoader cl=null;

	/**
	 * Throws an error if not called during DesignTime.
	 */
	public void setXFaceFactoryBeanName(String v)
		throws ClassNotFoundException, IOException
	{
		if (!Beans.isDesignTime())
			throw new Error("not DesignTime");
		xFaceFactoryBeanName=v;
		xFaceFactory=(XFaceFactory)Beans.instantiate(null,xFaceFactoryBeanName);
	}

	public String getXFaceFactoryBeanName()
	{
		return xFaceFactoryBeanName;
	}

	public XFace createXFace(XObj xObj)
		throws IOException, ClassNotFoundException
	{
		return xFaceFactory.createXFace(xObj);
	}

	public XFace createXFace(XObj xObj, ClassLoader cl)
		throws IOException, ClassNotFoundException
	{
		return xFaceFactory.createXFace(xObj,cl);
	}

	/**
	 * Throws an error if not called during DesignTime.
	 */
	public void setXObjFactoryBeanName(String v)
		throws ClassNotFoundException, IOException
	{
		if (!Beans.isDesignTime())
			throw new Error("not DesignTime");
		xObjFactoryBeanName=v;
		xObjFactory=(XObjFactory)Beans.instantiate(null,xObjFactoryBeanName);
	}

	public String getXObjFactoryBeanName()
	{
		return xObjFactoryBeanName;
	}

	public XObj createXObj()
	{
		return xObjFactory.createXObj();
	}

	public byte[] objectToBytes(Object o)
		throws IOException
	{
		if (o instanceof XFace)
		{
			String s=((XFace)o).getXObj().toString();
			return s.getBytes();
		}
		else
			throw new IOException("Object does not implement XFace");
	}

	public Object bytesToObject(byte[] b)
		throws IOException, ClassNotFoundException, NumberFormatException, Invalid64Exception
	{
		return bytesToObject(b,null);
	}
	public synchronized Object bytesToObject(byte[] b, ClassLoader cl)
		throws IOException, ClassNotFoundException, NumberFormatException, Invalid64Exception
	{
		this.b=b;
		this.cl=cl;
		off=0;
		int sz=b.length;
		while (off<sz && isWhiteSpace(b[off]))
			++off;
		if (off==sz)
			return null;
		if (b[off]!='<')
			throw new IOException("Unexpected byte at "+off);
		if (b[off+1]=='/')
			throw new IOException("Unexpected end tag at "+off);
		return parseElement();
	}

	private XFace parseElement()
		throws IOException, ClassNotFoundException, NumberFormatException, Invalid64Exception
	{
		int sz=b.length;
		++off;
		StringBuffer type=new StringBuffer();;
		while(!isDelim(b[off]))
			type.append((char)b[off++]);
		boolean empty=false;
		if (b[off]=='/')
		{
			empty=true;
			++off;
		}
		while (off<sz && isWhiteSpace(b[off]))
			++off;
		XObj xObj=createXObj();
		xObj.setType(type.toString());
		xObj.setEmpty(empty);
		while (b[off]!='>')
		{
			if (isDelim(b[off]))
				throw new IOException("Unexpected byte at "+off);
			StringBuffer att=new StringBuffer();
			while(!isDelim(b[off]))
				att.append((char)b[off++]);
			while (off<sz && isWhiteSpace(b[off]))
				++off;
			if (b[off]!='=')
				throw new IOException("= expected at "+off);
			++off;
			while (off<sz && isWhiteSpace(b[off]))
				++off;
			if (b[off]!='\"')
				throw new IOException("\" expected at "+off);
			++off;
			StringBuffer val=new StringBuffer();
			while(b[off]!='\"')
				val.append((char)b[off++]);
			++off;
			xObj.putAtt(att.toString(),val.toString());
			while (off<sz && isWhiteSpace(b[off]))
				++off;
		}
		++off;
		if (empty)
		{
			XFace rv=createXFace(xObj,cl);
			xObj.setXFace(rv);
			rv.setXObj(xObj);
			return rv;
		}
		Vector contents=xObj.getContents();
		while (true)
		{
			if (b[off]!='<')
			{
				StringBuffer txt=new StringBuffer();
				while(b[off]!='<')
					txt.append((char)b[off]);
				contents.addElement(txt.toString());
			}
			else if (b[off+1]!='/')
			{
				XFace xFace=parseElement();
				XObj xo=xFace.getXObj();
				contents.addElement(xo);
			}
			else
			{
				byte[] t=type.toString().getBytes();
				int at=off;
				off+=2;
				int i=0;
				for (i=0;i<t.length;++i)
					if (t[i]!=b[off++])
						throw new IOException("invalid end tag at "+at);
				if (b[off++]!='>')
					throw new IOException("invalid end tag at "+at);
				XFace rv=createXFace(xObj,cl);
				xObj.setXFace(rv);
				rv.setXObj(xObj);
				return rv;
			}
		}
	}

	private boolean isDelim(byte b)
	{
		return " \n\r\t<>=/".indexOf(b)!=-1;
	}

	private boolean isWhiteSpace(byte b)
	{
		return " \n\r\t".indexOf(b)!=-1;
	}
}